﻿// "Therefore those skilled at the unorthodox
// are infinite as heaven and earth,
// inexhaustible as the great rivers.
// When they come to an end,
// they bagin again,
// like the days and months;
// they die and are reborn,
// like the four seasons."
// 
// - Sun Tsu,
// "The Art of War"

using System;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using HtmlRenderer.Demo.Properties;

namespace HtmlRenderer.Demo
{
    public partial class BulkHtmlTestForm : Form
    {
        private const string CTesthtmls = @"C:\TestHtmls";

        public BulkHtmlTestForm()
        {
            InitializeComponent();

            Icon = Resources.html;

            OnLoadHtmlBttonClick(null, null);

            StartPosition = FormStartPosition.CenterScreen;
            var size = Screen.GetWorkingArea(Point.Empty);
            Size = new Size((int)(size.Width * 0.7), (int)(size.Height * 0.8));
        }

        private void OnLoadHtmlBttonClick(object sender, EventArgs e)
        {
            _htmlsListBox.Items.Clear();
            foreach (var htmlFile in Directory.GetFiles(CTesthtmls,"*.htm"))
            {
                var html = File.ReadAllText(htmlFile);
                var item = new ListItem(htmlFile, html);
                _htmlsListBox.Items.Add(item);
            }
        }

        private void OnHtmlsListBoxSelectedIndexChanged(object sender, EventArgs e)
        {
            var selectedItem = (ListItem) _htmlsListBox.SelectedItem;
            if(selectedItem != null)
            {
                var cssFilePath = Path.Combine(CTesthtmls, "Stylesheet.css");

                string bodyHtmlStyle = selectedItem.Html;
                if (File.Exists(cssFilePath))
                {
                    _htmlPanel.BaseStylesheet = File.ReadAllText(cssFilePath);
                    bodyHtmlStyle = string.Format("<link rel=\"Stylesheet\" href=\"{0}\" /> {1}", cssFilePath, selectedItem.Html);
                }

                _htmlPanel.Text = selectedItem.Html;
                _webBrowser.DocumentText = bodyHtmlStyle;
                
                PerformLayout();
                Invalidate();
            }
        }

        private void OnDeleteButtonClick(object sender, EventArgs e)
        {
            var selectedItem = (ListItem)_htmlsListBox.SelectedItem;
            if (selectedItem != null)
            {
                File.Delete(selectedItem.FullName);
                OnLoadHtmlBttonClick(null, null);
            }
        }

        private sealed class ListItem
        {
            public string FullName { get; private set; }
            public string Html { get; private set; }
            private string Name { get; set; }

            public ListItem(string fullName, string html)
            {
                FullName = fullName;
                Html = html;

                Name = Path.GetFileNameWithoutExtension(fullName);
            }

            public override string ToString()
            {
                return Name;
            }

        }
    }
}
